#include "GraphicsClass.h"


GraphicsClass::GraphicsClass(void)
	{
	m_D3D = 0;
	m_Camera = 0;
	m_Model = 0;
	 m_AxisModel = 0;
	m_ColorShader = 0;
	m_WaterModel = 0;
	
	}

GraphicsClass::GraphicsClass(const GraphicsClass&)
	{

	}
GraphicsClass::~GraphicsClass(void)
	{
	}

bool GraphicsClass:: Initialize(int screenWidth, int screenHeight, HWND hwnd)
	{
	bool result;

	//Ѿڣͷŵ
	Shutdown();

	// һD3DClass.
	m_D3D = new D3DClass;
	if(!m_D3D)
		{
		return false;
		}

	// D3DClassʼ
	result = m_D3D->Initialize(screenWidth, screenHeight, VSYNC_ENABLED, hwnd, FULL_SCREEN, SCREEN_DEPTH, SCREEN_NEAR);
	if(!result)
		{
		MessageBox(hwnd, L"Could not initialize Direct3D", L"Error", MB_OK);
		return false;
		}

	//
	m_Camera = new CameraClass;
	if(!m_Camera)
		{
		return false;
		}

	// λ.
	D3DXVECTOR3 campos = D3DXVECTOR3(8.0f, 3.0f, -20.0f);
	m_Camera->setPosition(&campos);

	// ģͶ.
	m_Model = new ModelClass;
	if(!m_Model)
		{
		return false;
		}

	// ʼģͶ.
	result = m_Model->Initialize(m_D3D->GetDevice(), 300, 300, 1.0f);
	if(!result)
		{
		MessageBox(hwnd, L"Could not initialize the model object.", L"Error", MB_OK);
		return false;
		}

	// ģͶ.
	m_AxisModel = new AxisModelClass;
	if(!m_AxisModel)
		{
		return false;
		}

	// ʼģͶ.
	result = m_AxisModel->Initialize(m_D3D->GetDevice());
	if(!result)
		{
		MessageBox(hwnd, L"Could not initialize the axis model object.", L"Error", MB_OK);
		return false;
		}

	// ˮģͶ.
	m_WaterModel = new WaterModelClass;
	if(!m_WaterModel)
		{
		return false;
		}

	// ʼˮģͶ.
	result = m_WaterModel->Initialize(m_D3D->GetDevice(),257, 257, 0.5f, 0.03f, 3.25f, 0.4f);
	if(!result)
		{
		MessageBox(hwnd, L"Could not initialize the water model object.", L"Error", MB_OK);
		return false;
		}

	// shader
	m_ColorShader = new ColorShaderClass;
	if(!m_ColorShader)
		{
		return false;
		}

	// ʼshader
	result = m_ColorShader->Initialize(m_D3D->GetDevice(), hwnd);
	if(!result)
		{
		MessageBox(hwnd, L"Could not initialize the color shader object.", L"Error", MB_OK);
		return false;
		}

	return true;
	}

void GraphicsClass::Shutdown()
	{
	// ͷshader.
	if(m_ColorShader)
		{
		m_ColorShader->Shutdown();
		delete m_ColorShader;
		m_ColorShader = 0;
		}

	// ͷģͶ.
	if(m_Model)
		{
		m_Model->Shutdown();
		delete m_Model;
		m_Model = 0;
		}
	// ͷģͶ.
	if(m_AxisModel)
		{
		m_AxisModel->Shutdown();
		delete m_AxisModel;
		m_AxisModel = 0;
		}

	// ͷˮģͶ.
	if(m_WaterModel)
		{
		m_WaterModel->Shutdown();
		delete m_WaterModel;
		m_WaterModel = 0;
		}

	// ͷ
	if(m_Camera)
		{
		delete m_Camera;
		m_Camera = 0;
		}

	//m_D3D
	if(m_D3D)
		{
		m_D3D->Shutdown();
		delete m_D3D;
		m_D3D = 0;
		}

	return;
	}


bool GraphicsClass::Frame(float dt)
	{
	bool result;


	// RenderȾ3D
	// RenderGraphicsClass˽к.
	result = Render(dt);
	if(!result)
		{
		return false;
		}

	return true;
	}


bool GraphicsClass::Render(float dt)
	{

	D3DXMATRIX viewMatrix, projectionMatrix, worldMatrix;
	bool result;


	// framebuffer.Ϊǳɫ
	m_D3D->BeginScene(0.0f, 0.0f, 0.5f, 1.0f);

	// õ3.
	m_Camera->getViewMatrix(&viewMatrix);
	m_D3D->GetWorldMatrix(worldMatrix);
	m_D3D->GetProjectionMatrix(projectionMatrix);

	//Ϊʵģʽ
	m_D3D->SetFillMode(D3D11_FILL_SOLID);

	m_AxisModel->Render(m_D3D->GetDeviceContext());
	// shaderȾ.
	result = m_ColorShader->Render(m_D3D->GetDeviceContext(), m_AxisModel->GetIndexCount(), worldMatrix, viewMatrix, projectionMatrix);
	if(!result)
		{
		return false;
		}    
	// ģͶߣ׼Ⱦ.
	m_Model->Render(m_D3D->GetDeviceContext());

	// shaderȾ.
	result = m_ColorShader->Render(m_D3D->GetDeviceContext(), m_Model->GetIndexCount(), worldMatrix, viewMatrix, projectionMatrix);
	if(!result)
		{
		return false;
		}

	static float t_base = 0.0f;
	static float total_time = 0.0f;
	total_time += dt;
	if( total_time - t_base >= 0.25f )
		{
		t_base += 0.25f;

		int i = 5 + rand() % 250;
		int j = 5 + rand() % 250;

		//õ12֮һ
		float r = 1.0 +  (float)(rand()) / (float)RAND_MAX*(2.0 - 1.0);

		m_WaterModel->disturb(i, j, r);
		}

	m_WaterModel->update(m_D3D->GetDeviceContext(), dt);
	//߿ģʽ
	m_D3D->SetFillMode(D3D11_FILL_WIREFRAME);
	// ģͶߣ׼Ⱦ.
	m_WaterModel->Render(m_D3D->GetDeviceContext());

	// shaderȾ.
	result = m_ColorShader->Render(m_D3D->GetDeviceContext(), m_Model->GetIndexCount(), worldMatrix, viewMatrix, projectionMatrix);
	if(!result)
		{
		return false;
		}

	//framebufferеͼpresentĻ.
	m_D3D->EndScene();

	return true;
	}
